@@ -23,7 +23,7 @@ use Archive::Builder::Generators ();
 # Version
 use vars qw{$VERSION};
 BEGIN {
-	$VERSION = 0.4;
+	$VERSION = '0.5';
 }
 
 
@@ -34,9 +34,9 @@ BEGIN {
 # Main Interface Methods
 
 # Constructor
-sub new { 
+sub new {
 	my $class = shift;
-	return bless { 
+	return bless {
 		sections => {},
 		}, $class;
 }
@@ -44,12 +44,12 @@ sub new {
 # Test generate and cache all files.
 sub test {
 	my $self = shift;
-	
+
 	# Run the test for each section.
 	foreach my $Section ( $self->section_list ) {
 		$Section->test or return undef;
 	}
-	
+
 	return 1;
 }
 
@@ -74,6 +74,18 @@ sub save {
 	return 1;
 }
 
+# Explicitly delete Archive
+sub delete {
+	my $self = shift;
+	
+	# Remove all our children
+	foreach ( $self->sections ) {
+		$_->delete;
+	}
+
+	return 1;
+}
+
 # Create a new archive for the Builder
 sub archive {
 	my $self = shift;
@@ -84,7 +96,7 @@ sub archive {
 # Create a more shorthand set of data, keying path against content ref
 sub _archive_content {
 	my $self = shift;
-	
+
 	# Get and merge the _archive_content()s for each section
 	my %tree = ();
 	foreach my $Section ( $self->section_list ) {
@@ -95,7 +107,7 @@ sub _archive_content {
 			$tree{$full} = $subtree->{$_};
 		}
 	}
-	
+
 	return \%tree;
 }
 
@@ -109,13 +121,13 @@ sub _archive_content {
 # Add a new section and return it
 sub new_section {
 	my $self = shift;
-	
+
 	# Create the section with the arguments
 	my $Section = Archive::Builder::Section->new( @_ )
 		or return undef;
-	
+
 	# Add the new section
-	return $self->add_section( $Section ) 
+	return $self->add_section( $Section )
 		? $Section : undef;
 }
 
@@ -124,15 +136,19 @@ sub add_section {
 	my $self = shift;
 	my $Section = isa( $_[0], 'Archive::Builder::Section' )
 		? shift : return undef;
-			
+
 	# Does a section with the name already exists?
 	my $name = $Section->name;
 	if ( exists $self->{sections}->{$name} ) {
 		return $self->_error( 'A section with that name already exists' );
 	}
-	
+
 	# Add the section
 	$self->{sections}->{$name} = $Section;
+
+	# Add it's parent reference
+	$Archive::Builder::Section::_PARENT{ refaddr $Section } = $self;
+	
 	return 1;
 }
 
@@ -154,14 +170,26 @@ sub section_list {
 sub section { $_[0]->{sections}->{$_[1]} }
 
 # Remove a section, by name
-sub remove_section { delete $_[0]->{sections}->{$_[1]} }
+sub remove_section {
+	my $self = shift;
+	my $name = $self->{sections}->{$_[0]} ? shift : return undef;
+	my $Section = $self->{sections}->{$name};
+	
+	# Delete from our sections
+	delete $self->{sections}->{$name};
+	
+	# Remove the parent link
+	delete $Archive::Builder::Section::_PARENT{ refaddr $Section };
 
-# Returns the number of files in the Builder, by totalling 
+	return 1;
+}
+
+# Returns the number of files in the Builder, by totalling
 # all it's sections
 sub file_count {
 	my $self = shift;
 	my $files = 0;
-	
+
 	foreach ( $self->section_list ) {
 		$files += $_->file_count;
 	}
@@ -169,7 +197,7 @@ sub file_count {
 	return $files;
 }
 
-	
+
 
 
 
@@ -180,59 +208,60 @@ sub _check {
 	my $either = shift;
 	my $type = shift;
 	my $string = shift;
-	
+
 	if ( $type eq 'name' ) {
 		return '' unless defined $string;
 		return $string =~ /^\w{1,31}$/ ? 1 : '';
-		
+
 	} elsif ( $type eq 'relative path' ) {
+		# This makes sure a directory isn't bad
 		return '' unless defined $string;
 		return '' unless length $string;
-		
+
 		# Get the canonical version of the path
 		my $canon = File::Spec->canonpath( $string );
-		
+
 		# Does the path contain escaping forward slashes
-		return '' if $string =~ /\\/;
-		
-		# Does the path contain relative directories
+		unless ( isa( 'File::Spec', 'File::Spec::Win32' ) ) {
+			return '' if $string =~ /\\/;
+		}
+
+		# Does the path contain upwards stuff?
+		return '' unless File::Spec->no_upwards( $string );
 		return '' if $string =~ /\.\./;
-		
-		# Does the path start with a slash?
-		return '' if $string =~ m!^/!;
-		
-		# Otherwise, looks good
-		return 1;				
-	
+
+		# Is the path absolute
+		return File::Spec->file_name_is_absolute( $string ) ? '' : 1;
+
 	} elsif ( $type eq 'generator' ) {
 		return $either->_error( 'No generator defined' ) unless defined $string;
-		
+
 		# Look for illegal characters
 		unless ( $string =~ /^\w+(::\w+)*$/ ) {
 			return $either->_error( 'Invalid function name format' );
 		}
 
-		# Is it a valid alias		
+		# Is it a valid alias
 		unless ( $string =~ /::/ ) {
 			$string = "Archive::Builder::Generators::$string";
 		}
-	
+
 		# All is good if the function is already loaded
 		{ no strict 'refs';
 			if ( defined *{"$string"}{CODE} ) {
 				return 1;
 			}
 		}
-	
+
 		# Does the class exist?
-		my ($module) = $string =~ m/^(.*)::.*$/;		
+		my ($module) = $string =~ m/^(.*)::.*$/;
 		unless ( Class::Inspector->installed( $module ) ) {
 			return $either->_error( "Package '$module' does not appear to be present" );
 		}
-		
+
 		# Looks good
 		return 1;
-		
+
 	} else {
 		return undef;
 	}
@@ -251,7 +280,7 @@ __END__
 
 =pod
 
-=head1 NAME 
+=head1 NAME
 
 Archive::Builder - File generation and archiving framework
 
@@ -287,24 +316,24 @@ varying types, or saving directly to disk.
 
 =head2 Structure
 
-Each C<Archive::Builder> object consists of one or more 
-C<Archive::Builder::Section>s, which contain one or more 
+Each C<Archive::Builder> object consists of one or more
+C<Archive::Builder::Section>s, which contain one or more
 C<Archive::Builder::File>s. Each of these files know their location
 within the section, and are given a generation function, with a set of
-arguments specific to each generator. Some simple generators are 
+arguments specific to each generator. Some simple generators are
 provided built-in, or you can provide an function name as the generator.
 
 =head2 Generating Archives
 
-Once a Archive::Builder is fully defined, you can C<save> it to disk, 
+Once a Archive::Builder is fully defined, you can C<save> it to disk,
 or get an C<archive>, containing the generated files, in one of several
 formats. You can also C<save> and get the C<archive> any of the individual
 sections within the builder.
 
 During the generation process for an entire C<Archive::Builder> a subdirectory
 is created for each section matching the name of the section. So, for a builder
-with a Section name 'one', containing a single file 'two.txt', and a section 
-'three', containing files 'four.html' and 'five.dat', the following file 
+with a Section name 'one', containing a single file 'two.txt', and a section
+'three', containing files 'four.html' and 'five.dat', the following file
 structure would result
 
   one/two.txt
@@ -319,18 +348,18 @@ the file being generated once, and the same contents returned twice.
 
 =head2 Generation on Demand Caveats
 
-During an output action, such as a C<save()> or C<archive()> method call, the 
-contents of each file are generated only as needed. This means that if the 
+During an output action, such as a C<save()> or C<archive()> method call, the
+contents of each file are generated only as needed. This means that if the
 generation of a file fails, an action may have already been taken ( especially
-in the case of C<save()>, where you may end up with only part of the files 
+in the case of C<save()>, where you may end up with only part of the files
 written to disk.
 
-Some procedures to address these limitations will be added as the module 
+Some procedures to address these limitations will be added as the module
 evolves.
 
 =head1 METHODS
 
-The methods for the three main classes involved in generation trees, 
+The methods for the three main classes involved in generation trees,
 C<Archive::Builder>, C<Archive::Builder::Section> and C<Archive::Builder::File>
 are documented below. For the archive handlers the builders generate, see
 L<Archive::Builder::Archive>. For information on the built-in generators, and
@@ -339,14 +368,14 @@ how to write your own generators, see L<Archive::Builder::Generators>.
 =head2 Common Error Handlers
 
 Errors from any object of any type below Archive::Builder are set in the global
-C<$Archive::Builder::errstr> variable. They can also be retrieved from the 
+C<$Archive::Builder::errstr> variable. They can also be retrieved from the
 C<errstr()> method called on any object.
 
 =head1 Archive::Builder
 
 =head2 new()
 
-The C<new()> constructor takes no arguments, and returns a new 
+The C<new()> constructor takes no arguments, and returns a new
 C<Archive::Builder> object.
 
 =head2 add_section( Archive::Builder::Section )
@@ -359,7 +388,7 @@ example if another Section with the same name has already been added.
 
 =head2 new_section( name )
 
-Creates a new C<Archive::Builder::Section> object with the name provided, and 
+Creates a new C<Archive::Builder::Section> object with the name provided, and
 immediately adds it to the builder.
 
 Returns the new Section object on success. Returns undef on error.
@@ -376,7 +405,7 @@ sections have been created in the builder.
 
 =head2 section_list()
 
-Returns all the sections in the builder as a list, sorted by section name. 
+Returns all the sections in the builder as a list, sorted by section name.
 Returns a null list C<()> if no sections are defined in the builder.
 
 =head2 remove_section( name )
@@ -391,16 +420,24 @@ Returns the total number of files in ass sections in the builder
 =head2 save( directory )
 
 Generates the file tree for the entire builder and attempts to save it
-below a given directory. The passed directory does not have to exist, it 
+below a given directory. The passed directory does not have to exist, it
 will be created on demand.
 
 Returns true if all files were generated and saved successfully. Returns
 C<undef> if an error occurs, or the directory is bad.
 
+=head2 delete()
+
+Because of the structure used to support the parent methods, you should 
+probably explicitly delete Builds when you are done with them to avoid
+memory leaks due to circular dependencies.
+
+The C<delete> method always returns true.
+
 =head2 archive( type )
 
 Creates a handle to an archive of a specified type related to the builder.
-Types can only be used if the modules that support them are installed. 
+Types can only be used if the modules that support them are installed.
 The following types are supported, and their prerequisites are listed.
 
   zip    - Archive::Zip
@@ -409,7 +446,7 @@ The following types are supported, and their prerequisites are listed.
   tgz    - Archive::Tar
 
 The tar.gz and tgz are aliases that produce the same thing with a different
-file extension. 
+file extension.
 
 The C<archive> method only returns a C<Archive::Builder::Archive> handle to
 the object, not the object itself. Also, the files are not generated at the
@@ -427,12 +464,12 @@ Creates a new C<Archive::Builder::Section> object of a given name. Although
 meant to be used in an C<Archive::Builder> object, they can still be used
 effectively standalone, as they have both C<save> and C<archive> methods.
 
-Returns undef is an invalid section name is given. A section name must 
+Returns undef is an invalid section name is given. A section name must
 contain only word ( \w ) characters and be 1 to 31 characters long.
 
-As a side note, the reason that Sections exist at all is so that Builders 
-can be defined containing multiple sections, where the sections will be 
-saved to different locations, but should still be passed around as a 
+As a side note, the reason that Sections exist at all is so that Builders
+can be defined containing multiple sections, where the sections will be
+saved to different locations, but should still be passed around as a
 single entity.
 
 =head2 name()
@@ -447,15 +484,22 @@ Builder root that will be used, or if passed a relative path, will set the
 path to a new value. You are not likely to need this, as in general, the
 same value will suffice for both the name and path.
 
+=head2 Builder()
+
+If the Section has been added to a Builder, the C<Builder> method will return
+it.
+
+Returns a C<Archive::Builder> object if added, or C<undef> if not.
+
 =head2 add_file( Archive::Builder::File )
 
 Adds an existing C<Archive::Builder::File> object to the section.
 
-Returns true on success. Returns C<undef> on error, or if the path 
-of the file clashes with an existing file in the Section. 
+Returns true on success. Returns C<undef> on error, or if the path
+of the file clashes with an existing file in the Section.
 
-This could happen if you try to add a file with the same name, of if your 
-path contains a directory that is already in the Section as a file. For 
+This could happen if you try to add a file with the same name, of if your
+path contains a directory that is already in the Section as a file. For
 example, the two files could not exist in the same Section.
 
   first/second
@@ -469,7 +513,7 @@ to find out.
 =head2 new_file( path, generator [, arguments ] )
 
 Creates a new file, using the arguments provided, and immediately adds it
-to the current section. See the C<new> method for Archive::Builder::File 
+to the current section. See the C<new> method for Archive::Builder::File
 below for more details on the arguments.
 
 Returns true if the file is created and added successfuly. Returns C<undef>
@@ -477,7 +521,7 @@ if an error occurs during either the creation or addition of the file.
 
 =head2 file( path )
 
-Finds the C<Archive::Builder::File> object with the given path and returns 
+Finds the C<Archive::Builder::File> object with the given path and returns
 it. Returns undef if no such file exists.
 
 =head2 files()
@@ -501,16 +545,23 @@ Returns the number of files contained in the section
 
 =head2 save( directory )
 
-The C<save()> method works the same as the C<Archive::Builder> C<save> method, 
-generating the files and saving them below the directory provided. Again, the 
+The C<save()> method works the same as the C<Archive::Builder> C<save> method,
+generating the files and saving them below the directory provided. Again, the
 directory is created on demand.
 
 Returns C<undef> if an error during generation or saving occurs.
 
+=head2 delete()
+
+The C<delete> method deletes a Section, removing it from it's parent Builder
+if applicable, and removing all child Files from the Section.
+
+Always returns true.
+
 =head2 archive( type )
 
 As for the C<Archive::Builder> C<acrhive> method, creates an archive handle of
-the given type. Returns a C<Archive::Builder::Archive> object on success. 
+the given type. Returns a C<Archive::Builder::Archive> object on success.
 Returns C<undef> on error.
 
 =head1 Archive::Builder::File
@@ -518,7 +569,7 @@ Returns C<undef> on error.
 =head2 new( path, generator [, arguments ] )
 
 Creates a new C<Archive::Builder::File> object and returns it. This method is
-not normally used directly, with the C<Archive::Builder::Section> method 
+not normally used directly, with the C<Archive::Builder::Section> method
 C<new_file()> being more typically used.
 
 The path argument should be a valid looking relative path. That is, it cannot
@@ -526,8 +577,8 @@ start with /. For safety, the use of escaping slashes and relative '..' paths
 are restricted for safety.
 
 The generator should be a string containing the name of the function to be used
-to generate the file contents. A check will be done to ensure that the module 
-containing the function is installed, although the existance of the function 
+to generate the file contents. A check will be done to ensure that the module
+containing the function is installed, although the existance of the function
 itself will not be tested. For example, for the generator function
 C<Foo::Bar::makeme>, a test to make sure C<Foo::Bar> is installed will be done.
 
@@ -538,7 +589,7 @@ of default generators, and instructions on how to write your own generators, are
 in the L<Archive::Builder::Generators> documentation.
 
 Anything passed after the generator are assumed to be arguments to the generator
-function, and will be stored and passed as needed. Note that the arguments are 
+function, and will be stored and passed as needed. Note that the arguments are
 not copied or cloned, so any objects passed as arguments and later modified, will
 be generated using the modified values. This is considered a feature. If you need
 to freeze a copy of the object for the generation, you are recommended to L<Clone>
@@ -554,23 +605,31 @@ Returns the generator for the file. This cannot be changed after creation
 
 =head2 arguments()
 
-Returns a reference to an array containing the arguments to be passed to 
+Returns a reference to an array containing the arguments to be passed to
 the generator, or C<0> if there are no arguments. The list of arguments
-cannot be changed after creation ( although of course objects passed can 
+cannot be changed after creation ( although of course objects passed can
 be changed outside the scope of this API ).
 
 =head2 binary()
 
-This method will analyse the file contents ( generating if needed ) to 
-determine if the file is a binary file. While not 100% accurate, it should 
+This method will analyse the file contents ( generating if needed ) to
+determine if the file is a binary file. While not 100% accurate, it should
 be good enough for most situations.
 
 =head2 executable()
 
-Calling this method will add a hint to the file that it should be considered 
-as an executable file, should the need arise. This is most likely used in 
+Calling this method will add a hint to the file that it should be considered
+as an executable file, should the need arise. This is most likely used in
 situations where permissions need to be set after generation.
 
+=head2 Section()
+
+If added to a Section, the C<Section> method returns the Section to which we
+have been added.
+
+Returns a L<Archive::Builder::Section> object if the File is added to one, or
+C<undef> if not.
+
 =head2 contents()
 
 Generates and returns the contents of the file as a scalar reference. Returns
@@ -578,11 +637,16 @@ C<undef> if a generation error occurs.
 
 =head2 save( filename )
 
-Bypassing the normal generation process and path name, the C<save> method 
-allows you to generate a single file object and save it to a specific 
+Bypassing the normal generation process and path name, the C<save> method
+allows you to generate a single file object and save it to a specific
 filename. Any directories need to write the file will be created on demand.
 Returns C<undef> if a generation permissions error occurs.
 
+=head2 delete()
+
+If added to a Section, the C<delete> method allows us to remove and delete the 
+file from the parent Section. Always returns true.
+
 =head1 TODO
 
 Better control over caching, more archive types, pre-generation testing.
@@ -596,7 +660,7 @@ Contact the author
         Adam Kennedy ( maintainer )
         cpan@ali.as
 	http://ali.as/
-        
+
 =head1 SEE ALSO
 
 L<Archive::Builder::Archive>, L<Archive::Builder::Generators>,
